import React, { useContext, useState, useEffect } from "react";
import { useNavigate } from "react-router-dom";
import { useJobCore, jobStatus } from "@qltr/jobcore";
import StoreContext from "@qltr/store";
import Box from "@mui/material/Box";
import AppBar from "@mui/material/AppBar";
import Toolbar from "@mui/material/Toolbar";
import IconButton from "@mui/material/IconButton";
import Typography from "@mui/material/Typography";
import Menu from "@mui/material/Menu";
import MenuItem from "@mui/material/MenuItem";

import JobLogView from "./JobLogView.jsx";
import ListItemText from "@mui/material/ListItemText";
import ListItemIcon from "@mui/material/ListItemIcon";
import MoreVertIcon from "@mui/icons-material/MoreVert";
import DownloadIcon from "@mui/icons-material/Download";
import ReplayIcon from "@mui/icons-material/Replay";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import DoNotDisturbIcon from "@mui/icons-material/DoNotDisturb";
import DeleteIcon from "@mui/icons-material/Delete";
import PlayArrowIcon from "@mui/icons-material/PlayArrow";
import ViewColumnIcon from "@mui/icons-material/ViewColumn";

export function useJobActionMenu(baseAnchor = null) {
  const [menuJob, setMenuJob] = useState({});
  const [anchorEl, setAnchorEl] = useState(baseAnchor);
  const menuOpen = (event) => setAnchorEl(event.currentTarget);
  const menuClose = () => {
    setAnchorEl(null);
    setMenuJob(null);
  };
  return [anchorEl, menuJob, setMenuJob, menuOpen, menuClose];
}

export default function JobActionMenu(props) {
  const { job, anchorEl, menuClose } = props;
  const { jobCompose, oneshotCancel, oneshotDestroy, toPipeline, toJob } =
    useJobCore();
  const { state: store } = useContext(StoreContext);

  const open = Boolean(anchorEl);

  function download(filename, text) {
    var element = document.createElement("a");
    element.setAttribute(
      "href",
      "data:text/plain;charset=utf-8," + encodeURIComponent(text)
    );
    element.setAttribute("download", filename);
    element.style.display = "none";
    document.body.appendChild(element);
    element.click();
    document.body.removeChild(element);
  }

  function retryJob() {
    const jobId = jobCompose(job.builderCache);
    if (store.focusJob) toJob(jobId);
  }

  function downloadLog() {
    if (job.status === jobStatus.PENDING) return;
    download(`${job.jobId}.txt`, job.log.join("\n"));
  }

  function cancelJob() {
    oneshotCancel(job.jobId);
  }

  function deleteJob() {
    oneshotDestroy(job.jobId);
    navigateToJobs();
  }

  const menuSelect = (cb) => () => {
    menuClose();
    cb();
  };

  function navigateToJobs() {
    if (job.isPipeline) return toPipeline(job.pipelineId);
    toJobs();
  }
  if (!job) return;
  return (
    <Menu anchorEl={anchorEl} open={open} onClose={menuClose}>
      <MenuItem onClick={menuSelect(downloadLog)}>
        <ListItemIcon>
          <DownloadIcon fontSize="small" />
        </ListItemIcon>
        <ListItemText>Download Log</ListItemText>
      </MenuItem>
      {!job.isPipeline && (
        <MenuItem onClick={menuSelect(retryJob)}>
          <ListItemIcon>
            {job.status === jobStatus.OK || job.status === jobStatus.ERROR ? (
              <ReplayIcon fontSize="small" />
            ) : (
              <PlayArrowIcon fontSize="small" />
            )}
          </ListItemIcon>
          <ListItemText>
            {job.status === jobStatus.ERROR ? "Retry" : "Duplicate"}
          </ListItemText>
        </MenuItem>
      )}
      {job.status === jobStatus.OK ||
      job.status === jobStatus.ERROR ||
      job.status === jobStatus.CANCELED ? null : (
        <MenuItem onClick={menuSelect(cancelJob)}>
          <ListItemIcon>
            <DoNotDisturbIcon fontSize="small" />
          </ListItemIcon>
          <ListItemText>Cancel</ListItemText>
        </MenuItem>
      )}
      {!job.isPipeline && (
        <MenuItem onClick={menuSelect(deleteJob)}>
          <ListItemIcon>
            <DeleteIcon fontSize="small" />
          </ListItemIcon>
          <ListItemText>Delete</ListItemText>
        </MenuItem>
      )}
    </Menu>
  );
}
